#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;

using System.IO;
using System.Net;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "HTTP", Category = "Network", Version = "REST Raw",
	Help = "posts raw data to given urls on the web", Author = "woei")]
	#endregion PluginInfo
	public class HttpPostRawNode : IPluginEvaluate, IPartImportsSatisfiedNotification
	{
		enum MethodType { GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS }
		#region fields & pins
		[Input("URL", StringType = StringType.URL, DefaultString = "http://localhost")]
		ISpread<string> FURL;
		
		[Input("Method")]
		ISpread<MethodType> FMethod;
		
		[Input("Content")]
		ISpread<System.IO.Stream> FContent;
		
		[Input("MimeType", EnumName = "MimeTypeMode")]
		ISpread<EnumEntry> FMimeType;
		
		[Input("Refresh", IsBang = true)]
		ISpread<bool> FRefresh;
		
		[Output("Status")]
		ISpread<string> FStatus;
		
		[Output("Header")]
		ISpread<string> FHeader;
		
		[Output("Body")]
		ISpread<System.IO.Stream> FResponse;
		
		[Output("Failed")]
		ISpread<bool> FFailed;
		
		[Output("Success")]
		ISpread<bool> FSuccess;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins

		public void OnImportsSatisfied()
		{
			FResponse.SliceCount = 0;
		}
		
		//called when data for any output pin is requested
		public void Evaluate(int spreadMax)
		{
			FResponse.ResizeAndDispose(spreadMax, () => new MemoryStream());
			FHeader.SliceCount = spreadMax;
			FStatus.SliceCount = spreadMax;
			FFailed.SliceCount = spreadMax;
			FSuccess.SliceCount = spreadMax;
			for (int i=0; i<spreadMax; i++)
			{
				if (FRefresh[i])
				{
					FHeader[i] = string.Empty;
					FResponse[i].SetLength(0);
					try
					{
						Uri url = new Uri(FURL[i]);
						HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
						request.Method = FMethod[i].ToString();
						request.Credentials = System.Net.CredentialCache.DefaultCredentials;
						
						byte[] buffer = new byte[1024];
						int bytesRead = 0;
						
						if (FContent[i].Length > 0)
						{
							request.ContentType = FMimeType[i].Name;
							request.ContentLength = FContent[i].Length;
							using (Stream reqstr = request.GetRequestStream())
							{
								while ((bytesRead = FContent[i].Read(buffer, 0, buffer.Length)) != 0) 
			            			reqstr.Write(buffer, 0, bytesRead);
		        			}
						}
						
						HttpWebResponse response = (HttpWebResponse)request.GetResponse();
						FStatus[i] = string.Format("{0} {1} {2} {3}", url.Scheme.ToUpper(), response.ProtocolVersion, (int)(response.StatusCode), response.StatusDescription);
						FHeader[i] = response.Headers.ToString();
						// Optional - use it if set by server
						if (response.ContentLength > 0)
							FResponse[i].SetLength(response.ContentLength);
						// Reset our stream position
						FResponse[i].Position = 0;
						using (Stream respstr = response.GetResponseStream())
						{							
							while ((bytesRead = respstr.Read(buffer, 0, buffer.Length)) != 0) 
		            			FResponse[i].Write(buffer, 0, bytesRead);
						}
						FResponse.Flush(true);
						FFailed[i] = false;
						FSuccess[i] = true;
					}
					catch (Exception e)
					{
						FLogger.Log(e);
						FFailed[i] = true;
						FSuccess[i] = false;
					}
				}
			}
		}
	}
}
